unit uConfig;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, INIFiles, ComCtrls, Registry, ShlObj, UPTShellUtils;

type
  TForm3 = class(TForm)
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Edit1: TEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Dir,
    Dir2: String;
  end;

var
  Form3: TForm3;

procedure RegisterFileType(cMyExt, cMyFileType, cMyDescription, ExeName: string;
  IcoIndex: integer; DoUpdate: boolean = false);

implementation

uses uMain;
{$R *.DFM}

procedure TForm3.FormCreate(Sender: TObject);
var
  INI: TMemIniFile;
  TreeNode: TTreeNode;
begin
  INI := TMemIniFile.Create(IncludeTrailingBackslash(ExtractFilePath(Application.Exename))+ 'rsbviewer.ini');
  try
    with INI do
    begin
      RadioButton1.Checked := ReadBool('settings', 'Remember', true);
      RadioButton2.Checked := not RadioButton1.Checked;
      Edit1.Text := ReadString('settings', 'UserDir',  '');

      if RadioButton1.Checked then
        Dir := ReadString('settings', 'LastDir',  '')
      else
        Dir := Edit1.text;
      if Dir = '' then
        Dir := ExtractFilePath(Application.Exename);
      IncludeTrailingBackslash(Dir);
    end;
    if frmMain.dlgSave.FileName = '' then
    begin
      frmMain.PTShellCombo1.SelectedFolder.PathName := Dir;
      //Form1.FolderTree1Change(self, TreeNode);
    end;
  finally
    INI.Free;
  end;
end;

procedure TForm3.FormDestroy(Sender: TObject);
var
  INI: TMemIniFile;
begin
  INI := TMemIniFile.Create(IncludeTrailingBackslash(ExtractFilePath(Application.Exename))+ 'rsbviewer.ini');
  try
    with INI do
    begin
      WriteBool('settings',   'Remember', RadioButton1.Checked);
      WriteString('settings', 'LastDir',  frmMain.FolderTree1.SelectedPathName);
      WriteString('settings', 'UserDir',  Edit1.text);
      UpdateFile;
    end;
  finally
    INI.Free;
  end;
end;

procedure RegisterFileType(cMyExt, cMyFileType, cMyDescription, ExeName: string;
  IcoIndex: integer; DoUpdate: boolean = false); 
var 
   Reg: TRegistry;
begin 
  Reg := TRegistry.Create; 
  try 
    Reg.RootKey := HKEY_CLASSES_ROOT; 
    Reg.OpenKey(cMyExt, True); 
    // Write my file type to it. 
    // This adds HKEY_CLASSES_ROOT\.abc\(Default) = 'Project1.FileType' 
    Reg.WriteString('', cMyFileType); 
    Reg.CloseKey; 
    // Now create an association for that file type 
    Reg.OpenKey(cMyFileType, True); 
    // This adds HKEY_CLASSES_ROOT\Project1.FileType\(Default) 
    //   = 'Project1 File' 
    // This is what you see in the file type description for 
    // the a file's properties. 
    Reg.WriteString('', cMyDescription); 
    Reg.CloseKey;    // Now write the default icon for my file type 
    // This adds HKEY_CLASSES_ROOT\Project1.FileType\DefaultIcon 
    //  \(Default) = 'Application Dir\Project1.exe,0' 
    Reg.OpenKey(cMyFileType + '\DefaultIcon', True); 
    Reg.WriteString('', ExeName + ',' + IntToStr(IcoIndex)); 
    Reg.CloseKey; 
    // Now write the open action in explorer 
    Reg.OpenKey(cMyFileType + '\Shell\Open', True); 
    Reg.WriteString('', '&Open'); 
    Reg.CloseKey; 
    // Write what application to open it with 
    // This adds HKEY_CLASSES_ROOT\Project1.FileType\Shell\Open\Command 
    //  (Default) = '"Application Dir\Project1.exe" "%1"' 
    // Your application must scan the command line parameters 
    // to see what file was passed to it. 
    Reg.OpenKey(cMyFileType + '\Shell\Open\Command', True); 
    Reg.WriteString('', '"' + ExeName + '" "%1"'); 
    Reg.CloseKey; 
    // Finally, we want the Windows Explorer to realize we added 
    // our file type by using the SHChangeNotify API. 
    if DoUpdate then SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil); 
  finally 
    Reg.Free; 
  end; 
end; 



procedure TForm3.BitBtn1Click(Sender: TObject);
begin
  RegisterFileType('.rsb', 'RSBfile', 'Red Storm Bitmap', Application.ExeName, 1, true);
  ShowMessage('Done...');
end;

procedure TForm3.Button1Click(Sender: TObject);
var
  lpItemID : PItemIDList;
  bi : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  FillChar(bi, sizeof(TBrowseInfo), #0);
  bi.hwndOwner := Handle;
  bi.pszDisplayName := @DisplayName;
  bi.lpszTitle := PChar('Select directory');
  bi.ulFlags := BIF_RETURNONLYFSDIRS;
  lpItemID := SHBrowseForFolder(bi);

  if lpItemId <> nil then
  begin
    SHGetPathFromIDList(lpItemID, TempPath);
    Edit1.text := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;



end.


