//
// RSB Viewer
// Copyright (C) Dan Strandberg 2000, 2001
//
unit uMain;

interface

uses
  Windows, ComCtrls, Controls, ExtCtrls, Classes, Forms, SysUtils, StdCtrls,
  Graphics, Menus, ToolWin, Dialogs, ImgList, Buttons, UPTTreeList,
  UPTShellControls, UPTImageCombo, UPTShellUtils,
  ShellApi,   // DragAcceptFiles()...
  Math,       // Power()
  Messages,   // TWM_DROPFILES, WM_DROPFILES
  FileCtrl;   // DirectoryExists...


type
  TRSBHeader = packed record
    version : integer;
    width   : integer;
    height  : integer;
  end;

  TRsbBitInfo = packed record
    red_bits   : integer;
    green_bits : integer;
    blue_bits  : integer;
    alpha_bits : integer;
  end;

  PByte = ^Byte;

  TfrmMain = class(TForm)
    stbMain: TStatusBar;
    splMain: TSplitter;
    Panel1: TPanel;
    sbxMain: TScrollBox;
    imgMain: TImage;
    Splitter1: TSplitter;
    dlgSave: TSaveDialog;
    Panel2: TPanel;
    ToolBar: TToolBar;
    tbn1: TToolButton;
    btnAbout: TSpeedButton;
    btnSave: TSpeedButton;
    btnConfig: TSpeedButton;
    tbn2: TToolButton;
    ImageList2: TImageList;
    FolderTree1: TPTShellTree;
    PTShellCombo1: TPTShellCombo;
    PTShellList1: TPTShellList;
    procedure FormCreate(Sender: TObject);
    procedure FolderTree1Change(Sender: TObject; Node: TTreeNode);
    procedure ToolButton1Click(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure cbDoCheckClick(Sender: TObject);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure btnConfigClick(Sender: TObject);
    procedure imgMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgMainMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure sbxMainResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure stbMainDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
  private
    procedure ViewRSB(RSB: String);
    procedure ViewBMP(BMP: String);
    procedure ClearImage;
    procedure DropFiles(var msg : TWmDropFiles); message WM_DROPFILES;
    { Private declarations }
  public
    { Public declarations }
  end;

const
  VERSION = '1.4';            // App version
  crHand  = 1;                // Custom cursor 'HAND' by C_A_P

var
  frmMain: TfrmMain;
  Aimage : TBitmap;           // Used internally to draw on.
  RSBFile: TFileStream;
  Header: TRSBHeader;         //  RSB Header crap
  HasPal: LongBool;           //      -- " --
  BitInf: TRsbBitInfo;        //      -- " --
  StartX, StartY : Integer;   //  When dragging images. startpos is stored here.
  Move : Boolean = false;     //  Move the image (scroll) ?
  SmallList: TImageList;      //  Sysem imagelist

implementation

uses uAbout, uConfig;         // Config and about form.

{$R *.DFM}
{$R icons.res}                // Custom resource.. Contains the 'RSB' icon and
                              // C_A_P's hand cursor.


procedure TfrmMain.ViewRSB(RSB: String);
var
  X,Y      : Integer;        // For :=... counters.
  C        : ^Word;          // Current 16-bit pixel
  D        : PByte;          // Destination (decoded data, 24-bit pixels)
  MemStr   : TMemoryStream;  // The file
  S        : Pointer;        // Source (16-bit pixels)
  B_Sh,                      // Blue shift
  G_Sh,                      // Green shift
  R_Sh,                      // Red shift
  G_Pw,                      // Green power
  R_Pw     : Byte;           // Red power
  t1,                        // Start time
  t2       : Single;         // End time
begin
  stbMain.Panels.Items[0].Text := 'Loading...';
  stbMain.Invalidate;

  // Get start time!
  t1 := GetTickCount;

  // Open files. MemoryStreams are faster than FileStreams and Memstrms can load
  // files even if they are 'locked' by other programs.
  MemStr := TMemoryStream.Create;
  try
    try
      MemStr.LoadFromFile(RSB);
      MemStr.Seek(0, soFromBeginning);
      MemStr.ReadBuffer(Header, SizeOf(Header));

      if not (header.version in[0,1]) then
        raise Exception.Create('Unsuported format or version!');

      imgMain.Picture.Bitmap.Width  := Header.width;  // TImage
      imgMain.Picture.Bitmap.Height := header.height;
      imgMain.Width  := Header.width;  // TImage
      imgMain.Height := header.height;

      // If version is 0 then read a 4byte boolean (LongBool) to see if it has
      // 8-bit image data in it aswell.
      if Header.Version = 0 then
      begin
        MemStr.Read(HasPal, SizeOf(HasPal));
        if HasPal then
          MemStr.Seek(1024 + Header.width*Header.height, soFromCurrent);
      end;

      // Read color bit info.
      MemStr.Read(BitInf, SizeOf(BitInf));

      B_Sh := (8 - BitInf.blue_bits                                            );
      G_Sh := (    BitInf.green_bits + BitInf.blue_bits                     - 8);
      R_Sh := (    BitInf.red_bits   + BitInf.green_bits + BitInf.blue_bits - 8);

      G_Pw := (256 - trunc(power(2, 8 - BitInf.blue_bits)));
      R_Pw := (256 - trunc(power(2, 8 - BitInf.red_bits )));

      // Get mem for pixels
      GetMem(S, Header.Height*Header.Width*2);

      // Point the 'decode pixel' to it.
      C := S;

      // put source pixels in memory.. to speed things up.
      MemStr.Read(S^, Header.Height*Header.Width*2);

      // Get rid of the RSB file.. we don't need it anomore.
      FreeAndNil(MemStr);

      for Y := 0 to header.height -1 do
      begin
        D := imgMain.Picture.Bitmap.Scanline[y];
        For X := 0 to Header.width  -1 do
        begin
          D^ := C^ shl B_Sh;            // Blue
          Inc(Integer(D),1);

          D^ := (C^ shr G_Sh) and G_Pw; // Green
          Inc(Integer(D),1);

          D^ := (C^ shr R_Sh) and R_Pw; // Red
          Inc(Integer(D),1);

          // Note I don't "extract" the alpha since I don't use it.

          // Point to next 16-bit pixel
          Inc(Integer(C),2);
        end;
      end;

      // Repaint otherwise the image might now show!
      imgMain.Repaint;

      // Image displayed.. get end time!
      t2 := GetTickCount;

      // Set default savename
      dlgSave.FileName := ChangeFileExt(RSB,'.bmp');

      // Enable save button
      btnSave.Enabled := True;

      stbMain.Panels.Items[0].Text := ExtractFileName(RSB);
      stbMain.Panels.Items[1].Text := IntToStr(Header.width) + ' x ' + IntToStr(Header.height);


      stbMain.Panels.Items[2].Text := '16-bit ('+IntToStr(BitInf.red_bits) +'-'+ IntToStr(BitInf.green_bits) +'-'+ IntToStr(BitInf.blue_bits) +'-'+ IntToStr(BitInf.alpha_bits) +')';
      stbMain.Panels.Items[3].Text := FloatToStrF((t2-t1)/1000,ffFixed,7,2)+' s';


    except
      on E: Exception do
      begin
        ClearImage;
        stbMain.Panels.Items[0].Text := ExtractFileName(RSB);
        stbMain.Panels.Items[1].Text := '';
        stbMain.Panels.Items[2].Text := '';
        stbMain.Panels.Items[3].Text := E.Message;
      end;
    end;
  finally
    If Assigned(MemStr) then
      FreeAndNil(MemStr);
    If Assigned(S) then
      FreeMem(S);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  i: integer;
  Tmp: String;
begin
  // Set caption / Application title
  Caption := Format('RSB Viewer %s', [VERSION]);
  Application.Title := Caption;

  // Create a bitmap
  imgMain.Picture.Bitmap := TBitmap.Create;
  imgMain.Picture.Bitmap.PixelFormat :=pf24bit;
  
  // Make program accept "drag files"
  DragAcceptFiles(Handle, True);

  // Give the Form OnKeyPress a peak at all keys pressed in the app.
  KeyPreview := True;

  // Load C_A_P's hand cursor
  Screen.Cursors[crHand] := LoadCursor(hInstance, 'HAND');

  // Get system-images
  SmallList := TImageList.Create(frmMain);
  // DO THIS or we'll free windows imagelist when we close the app! And that's NOT good.
  SmallList.ShareImages := true;  
  SmallList.Handle := ShellGetSystemImageList(ptsizSmall);
  SmallList.DrawingStyle := dsTransparent;

  // Started with parameters?
  if ParamCount > 0 then
  begin
    // Parameter is not an excisting file!
    If not FileExists(paramstr(1)) then
      exit;

    PTShellCombo1.SelectedFolder.PathName := ExtractFilePath(paramstr(1));
    Tmp := ExtractFileName(paramstr(1));

    //FindRsbFiles(ExtractFilePath(paramstr(1))); - OLD
    for i := 0 to PTShellList1.Items.Count-1 do
    begin
      // this will trigger ListView1.OnSelectItem and show the image...
      PTShellList1.Items.Item[i].Selected := (PTShellList1.ShListData[i].FileName = Tmp);
    end;
  end;
end;

// Clears the image vissible image.
procedure TfrmMain.ClearImage;
begin
  if not Assigned(AImage) then
    AImage := TBitmap.Create;

  imgMain.Picture.Bitmap := AImage;
  imgMain.Picture.Bitmap.PixelFormat :=pf24bit;

  dlgSave.FileName := '';
  btnSave.Enabled := false;

  stbMain.Panels[1].Text := '';
  stbMain.Panels[2].Text := '';
  stbMain.Panels[3].Text := '';

end;

procedure TfrmMain.FolderTree1Change(Sender: TObject; Node: TTreeNode);
begin
  ClearImage;
  // Find files in the new directory
  //FindRSBFiles(FolderTree1.SelectedPathName); - OLD
end;

// Save open image
procedure TfrmMain.ToolButton1Click(Sender: TObject);
begin
  if (dlgSave.FileName <> '') and (PTShellList1.SelCount > 0) and (imgMain.Picture.Bitmap.Width > 0) then
  begin
    if not dlgSave.Execute then Exit;
    imgMain.Picture.SaveToFile(dlgSave.Filename);
  end;
end;

// Show aboutbox
procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

procedure TfrmMain.cbDoCheckClick(Sender: TObject);
begin

end;

// View selected image. Since I designed the program to only view rsb at first
// the abillity to view bmp is just a hack... so please excuse the crappy code
// for that.
procedure TfrmMain.ListView1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
var
  Ext: String;
begin
  if (Selected) then
  begin
    Ext := UPPERCASE(ExtractFileExt(PTShellList1.SelectedItem.FileName));
    if FileExists(IncludeTrailingBackslash(FolderTree1.SelectedPathName) + PTShellList1.SelectedItem.FileName) then
    begin
      if Ext = '.RSB' then
        ViewRSB(IncludeTrailingBackslash(FolderTree1.SelectedPathName) + PTShellList1.SelectedItem.FileName)
      else
        if Ext = '.BMP' then
          ViewBMP(IncludeTrailingBackslash(FolderTree1.SelectedPathName) + PTShellList1.SelectedItem.FileName);
    end;
  end;
end;

// Close program if 'Escape' is pressed.
procedure TfrmMain.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then Close;
end;

// Show config
procedure TfrmMain.btnConfigClick(Sender: TObject);
begin
  Form3.ShowModal;
end;

procedure TfrmMain.DropFiles(var msg: TWmDropFiles);
var
  n,i,
  nFiles: Integer;
  Filename:String;
begin
  nFiles := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
  try
    for i := nFiles-1 downto 0 do
    begin
      SetLength(FileName, 80);
      DragQueryFile(Msg.Drop, i, PChar(FileName), 80);
      FileName := PChar(Filename);
      if (UPPERCASE(ExtractFileExt(Filename)) = '.RSB') or
        (UPPERCASE(ExtractFileExt(Filename)) = '.BMP') then
      begin
        If not FileExists(Filename) then
          continue;
        PTShellCombo1.SelectedFolder.PathName := ExtractFilePath(Filename);
        //FindRsbFiles(ExtractFilePath(Filename));
        for n := 0 to PTShellList1.Items.Count-1 do
        begin
          {this will trigger ListView1.OnSelectItem and show the image...}
          PTShellList1.Items.item[n].Selected := (PTShellList1.Items.item[n].Caption = ExtractFileName(Filename));
        end;
        Exit;
      end;
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TfrmMain.imgMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  // Start moving image
  Move := True;
  StartX := X;
  StartY := Y;
end;

procedure TfrmMain.imgMainMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Move := False;
end;

procedure TfrmMain.imgMainMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if Move then
    with sbxMain do
    begin
      VertScrollBar.Position := VertScrollBar.Position - (Y - StartY);
      HorzScrollBar.Position := HorzScrollBar.Position - (X - StartX);
      imgMain.Refresh;
    end;
end;

procedure TfrmMain.sbxMainResize(Sender: TObject);
begin
  if (sbxMain.width < imgMain.width) or (sbxMain.Height < imgMain.Height) then
    imgMain.Cursor := crHand
  else
    imgMain.Cursor := crDefault;
end;

procedure TfrmMain.ViewBMP(BMP: String);
var
  t1, t2: single;
begin
  stbMain.Panels.Items[0].Text := 'Loading...';
  stbMain.Panels.Items[1].Text := '';
  stbMain.Panels.Items[2].Text := '';
  stbMain.Panels.Items[3].Text := '';
  stbMain.Invalidate;
  t1 := GetTickCount;
  try
    // LoadFromFile will have to do for now...
    imgMain.Picture.LoadFromFile(BMP);
    imgMain.Width  := imgMain.Picture.Bitmap.width;
    imgMain.Height := imgMain.Picture.Bitmap.height;
    t2 := GetTickCount;
    stbMain.Panels.Items[0].Text := ExtractFileName(BMP);
    stbMain.Panels.Items[1].Text := IntToStr(imgMain.width) + ' x ' + IntToStr(imgMain.height);
    case imgMain.Picture.Bitmap.PixelFormat of
      pf1bit : stbMain.Panels.Items[2].Text := '1-bit';
      pf4bit : stbMain.Panels.Items[2].Text := '4-bit';
      pf8bit : stbMain.Panels.Items[2].Text := '8-bit';
      pf15bit : stbMain.Panels.Items[2].Text := '15-bit';
      pf16bit : stbMain.Panels.Items[2].Text := '16-bit';
      pf24bit : stbMain.Panels.Items[2].Text := '24-bit';
      pf32bit : stbMain.Panels.Items[2].Text := '32-bit';
    end;
    stbMain.Panels.Items[3].Text := FloatToStrF((t2-t1)/1000,ffFixed,7,2)+' s';
    btnSave.Enabled := False;
  except
    // Error? Show error message in the statusbar.
    on E: Exception do
    begin
      ClearImage;
      stbMain.Panels.Items[0].Text := ExtractFileName(BMP);
      stbMain.Panels.Items[3].Text := E.message ;
    end;
  end;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(AImage) then
    AImage.Free;
  if Assigned(SmallList) then
    FreeAndNil(SmallList);
end;

// Draw images and text on the statusbar...
procedure TfrmMain.stbMainDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  if Panel.Index = 1 then
    ImageList2.Draw(stbMain.Canvas, Rect.Left+1, Rect.Top, 0);
  if Panel.Index = 2 then
    ImageList2.Draw(stbMain.Canvas, Rect.Left, Rect.Top, 1);
  if Panel.Index = 3 then
    ImageList2.Draw(stbMain.Canvas, Rect.Left, Rect.Top, 2);

  if (PTShellList1.SelCount = 1) then
    Case Panel.Index of
      0 :
      begin
        SmallList.Draw(stbMain.Canvas,Rect.Left+1, Rect.Top,
          ShellGetIconIndex(PTShellList1.SelectedItem.AbsoluteIdList,
          SHGFI_SMALLICON));
          stbMain.canvas.TextOut(Rect.left+18, Rect.top+2, Panel.Text);
      end;
      1 : stbMain.canvas.TextOut(Rect.left+18, Rect.top+2, Panel.Text);
      2 : stbMain.canvas.TextOut(Rect.left+17, Rect.top+2, Panel.Text);
      3 : stbMain.canvas.TextOut(Rect.left+17, Rect.top+2, Panel.Text);
    end;
end;

end.
